"""
Configuration of Doconce parameters that can also be set
on the command line.
For example, a line in this file:

some_option = some_value

corresponds to a command-line option --some_option=some_value
"""

help = False
debug = False
no_abort = False
#verbose =
#language =
preprocess_include_subst = False
#syntax_check =
skip_inline_comments = False
draft = False
#CC_license =
align2equations = False
force_tikz_conversion = False
#tikz_libs =
IBPLOT = False
#exercise_numbering =
exercises_in_zip = False
#exercises_in_zip_filename =
#toc_depth =
#encoding =
no_ampersand_quote = False
no_mako = False
no_preprocess = False
mako_strict_undefined = False
no_header_footer = False
no_emoji = False
siunits = False
allow_refs_to_external_docs = False
runestone = False
#max_bc_linelength =
#replace_ref_by_latex_auxno =
keep_pygments_html_bg = False
#minted_latex_style =
#pygments_html_style =
pygments_html_linenos = False
xhtml = False
#html_output =
#html_style =
#html_template =
no_title = False
#html_code_style =
#html_pre_style =
#html_toc_indent =
#html_body_style =
#html_body_font =
#html_heading_font =
#html_video_autoplay =
#html_admon =
html_admon_shadow = False
#html_admon_bg_color =
#html_admon_bd_color =
#css =
html_box_shadow = False
#html_share =
#html_exercise_icon =
#html_exercise_icon_width =
#html_raw_github_url =
html_DOCTYPE = False
html_links_in_new_window = False
#html_quiz_button_text =
#html_bootstrap_navbar =
#html_bootstrap_jumbotron =
#html_bootstrap_navbar_links =
#html_figure_caption =
#html_figure_hrule =
#html_copyright =
html_responsive_figure_width = False
cite_doconce = False
#device =
number_all_equations = False
denumber_all_equations = False
#latex_style =
#latex_font =
#latex_code_style =
#latex_code_leftmargin =
#latex_code_bg =
latex_code_bg_vpad = False
#latex_code_lststyles =
#latex_copyright =
#latex_bibstyle =
#section_numbering =
#latex_table_format =
#latex_table_row_sep =
#latex_title_layout =
#latex_link_color =
#latex_title_reference =
#latex_encoding =
#latex_packages =
#latex_papersize =
#latex_list_of_exercises =
#latex_movie =
#latex_movie_controls =
latex_external_movie_viewer = False
latex_fancy_header = False
#latex_section_headings =
#latex_colored_table_rows =
latex_line_numbers = False
latex_todonotes = False
latex_double_spacing = False
latex_labels_in_margin = False
latex_index_in_margin = False
#latex_preamble =
latex_no_program_footnotelink = False
#latex_admon =
#latex_admon_color =
latex_admon_title_no_period = False
#latex_admon_envir_map =
#latex_subex_header_postfix =
xelatex = False
latex_double_hyphen = False
#latex_elsevier_journal =
#ipynb_version =
#ipynb_split_pyshell =
#ipynb_cite =
#ipynb_admon =
#ipynb_figure =
#ipynb_movie =
verbose = False
examples_as_exercises = False
exercises_as_subsections = False
solutions_at_end = False
without_solutions = False
answers_at_end = False
without_answers = False
without_hints = False
#exercise_solution =
wordpress = False
tables2csv = False
sections_up = False
sections_down = False
#os_prompt =
#code_skip_until =
#code_prefix =
#figure_prefix =
#movie_prefix =
no_mp4_webm_ogg_alternatives = False
handout = False
urlcheck = False
#labelcheck =
#short_title =
markdown = False
#md2do_output =
github_md = False
slate_md = False
strapdown = False
#bootswatch_theme =
strict_markdown_output = False
multimarkdown_output = False
#quiz_question_prefix =
#quiz_choice_prefix =
#quiz_horizontal_rule =
#quiz_explanations =
rst_uio = False
rst_mathjax = False
sphinx_keep_splits = False
#sphinx_figure_captions =
oneline_paragraphs = False



from __future__ import absolute_import

from .epytext import epytext_author
from .common import default_movie, plain_exercise, DEFAULT_ARGLIST

def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           EXERCISE,
           FIGURE_EXT,
           CROSS_REFS,
           INDEX_BIB,
           TOC,
           ENVIRS,
           QUIZ,
           INTRO,
           OUTRO,
           filestr):
    # all arguments are dicts and accept in-place modifications (extensions)

    FILENAME_EXTENSION['st'] = '.st'
    BLANKLINE['st'] = '\n'
    # replacement patterns for substitutions of inline tags
    INLINE_TAGS_SUBST['st'] = {
        'math':      r'\g<begin>\g<subst>\g<end>',
        'math2':     r'\g<begin>\g<puretext>\g<end>',
        'emphasize': None,
        'bold':      r'\g<begin>**\g<subst>**\g<end>',
        'verbatim':  r"\g<begin>'\g<subst>'\g<end>",
        #'linkURL':   r'\g<begin>"\g<url>":\g<link>\g<end>',
        'linkURL2':  r'"\g<url>":\g<link>',
        'linkURL3':  r'"\g<url>":\g<link>',
        'linkURL2v': r"""\g<url>:'\g<link>'""",
        'linkURL3v': r"""\g<url>:'\g<link>'""",
        'plainURL':  r'"\g<url>":\g<url>',
        'colortext': '\g<text>',
        # the replacement string differs, depending on the match object m:
        'chapter':       r'\g<subst>',
        'section':       r'\g<subst>',
        'subsection':    r'\g<subst>',
        'subsubsection': r'\g<subst>\n',
        'paragraph':     r'*\g<subst>*\g<space>',  # extra blank
        'abstract':      r'*\g<type>.* \g<text>\n\g<rest>',
        'title':         r'TITLE: \g<subst>',
        'date':          r'DATE: \g<subst>',
        'author':        epytext_author,
        'movie':         default_movie,
        'linebreak':     r'\g<text>',
        'non-breaking-space': ' ',
        'ampersand2':    r' \g<1>&\g<2>',
        }

    from .rst import rst_code, rst_table
    CODE['st'] = rst_code
    TABLE['st'] = rst_table

    LIST['st'] = {
        'itemize':
        {'begin': '', 'item': '-', 'end': '\n'},

        'enumerate':
        {'begin': '', 'item': '%d.', 'end': '\n'},

        'description':
        {'begin': '', 'item': '%s -- ', 'end': '\n'},

        'separator': '',
        }
    ARGLIST['st'] = DEFAULT_ARGLIST
    from .plaintext import plain_ref_and_label, plain_index_bib
    CROSS_REFS['st'] = plain_ref_and_label
    INDEX_BIB['st'] = plain_index_bib
    EXERCISE['st'] = plain_exercise
    TOC['st'] = lambda s, f: ''  # drop
    from .plaintext import plain_quiz
    QUIZ['st'] = plain_quiz



from __future__ import absolute_import
import re
from .common import indent_lines, default_movie, plain_exercise


def epytext_author(authors_and_institutions, auth2index,
                   inst2index, index2inst, auth2email):
    text = 'BY: '
    ai = []
    for author, institutions, email in authors_and_institutions:
        if institutions is not None:
            ai.append('%s (%s)' % (author, ', and '.join(institutions)))
        else:
            ai.append(author)
    text += '; '.join(ai)
    return text


def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           EXERCISE,
           FIGURE_EXT,
           CROSS_REFS,
           INDEX_BIB,
           TOC,
           ENVIRS,
           QUIZ,
           INTRO,
           OUTRO,
           filestr):
    # all arguments are dicts and accept in-place modifications (extensions)

    FILENAME_EXTENSION['epytext'] = '.epytext'
    BLANKLINE['epytext'] = '\n'
    # replacement patterns for substitutions of inline tags
    INLINE_TAGS_SUBST['epytext'] = {
        'math':      r'\g<begin>M{\g<subst>}\g<end>',
        #'math2':     r'\g<begin>M{\g<latexmath>}\g<end>', # has \ in \sin e.g. so we rather use the puretext version
        'math2':    r'\g<begin>M{\g<puretext>}\g<end>',
        'emphasize': r'\g<begin>I{\g<subst>}\g<end>',
        'bold':      r'\g<begin>B{\g<subst>}\g<end>',
        'verbatim':  r'\g<begin>C{\g<subst>}\g<end>',
        #'linkURL':   r'\g<begin>U{\g<link><\g<url>>}\g<end>',
        'linkURL2':  r'U{\g<link><\g<url>>}',
        'linkURL3':  r'U{\g<link><\g<url>>}',
        'linkURL2v': r'U{C{\g<link>}<\g<url>>}',
        'linkURL3v': r'U{C{\g<link>}<\g<url>>}',
        'plainURL':  r'U{\g<url><\g<url>>}',
        'colortext': '\g<text>',
        # the replacement string differs, depending on the match object m:
        'chapter':       lambda m: '%s\n%s' % (m.group('subst'), '%'*len(m.group('subst'))),
        'section':       lambda m: '%s\n%s' % (m.group('subst'), '='*len(m.group('subst'))),
        'subsection':    lambda m: '%s\n%s' % (m.group('subst'), '-'*len(m.group('subst'))),
        'subsubsection': lambda m: '%s\n%s\n' % (m.group('subst'), '~'*len(m.group('subst'))),
        'paragraph':     r'I{\g<subst>}\g<space>',
        'abstract':      r'\nI{\g<type>.} \g<text>\n\g<rest>',
        'title':         r'TITLE: \g<subst>',
        'date':          r'DATE: \g<subst>',
        'author':        epytext_author,
        'movie':         default_movie,
        'linebreak':     r'\g<text>',
        'non-breaking-space': ' ',
        'ampersand2':    r' \g<1>&\g<2>',
        }

    from .rst import rst_code, rst_table
    CODE['epytext'] = rst_code
    TABLE['epytext'] = rst_table
    from .plaintext import plain_ref_and_label, plain_index_bib
    CROSS_REFS['epytext'] = plain_ref_and_label
    INDEX_BIB['epytext'] = plain_index_bib
    EXERCISE['epytext'] = plain_exercise

    LIST['epytext'] = {
        'itemize':
        {'begin': '', 'item': '-', 'end': '\n'},

        'enumerate':
        {'begin': '', 'item': '%d.', 'end': '\n'},

        'description':
        {'begin': '', 'item': '%s', 'end': '\n'},

        'separator': '',
        }
    ARGLIST['epytext'] = {
        'parameter': '@param',
        'keyword': '@keyword',
        'return': '@return',
        'instance variable': '@ivar',
        'class variable': '@cvar',
        'module variable': '@var',
        }
    TOC['epytext'] = lambda x, f: '\n'  # no toc for epydoc
    from .plaintext import plain_quiz
    QUIZ['epytext'] = plain_quiz

    #insert QUIZ in various .py files, plaintext can do something simple, problem: cannot insert headline and exercise because then the exercise is not interpreted! problem2: quiz will just be a part of an exercises, unrendered
    #solution problem1: typeset_quizzes1 inserts the headline if the first headline prior to the quiz is not Exercise: heading, if heading is missing see if it can be obtained from the last Exercise|Project|Problem heading (if last heading is of this type)
    #solution problem2: the exercise data struct is not written to file before a similar interpretation as in extract_quizzes, then we can drop substituting in the data structure too!



"""
MediaWiki translator, aimed at Wikipedia/WikiBooks type of web pages.
Syntax defined by http://en.wikipedia.org/wiki/Help:Wiki_markup
and http://en.wikipedia.org/wiki/Help:Displaying_a_formula.
The prefix m in the name mwiki distinguishes this translator from
gwiki (googlecode wiki).

Not yet implemented:
mwiki_ref_and_label (just using code from gwiki)

Just using plan ASCII solutions for index_bib (requires some work to
port to MediaWiki, but is straightforward - use rst as template) and
exercise (probably ok with the plain solution).

GitHub wiki pages understand MediaWiki, see
https://github.com/github/gollum

The page http://en.wikibooks.org/wiki/Wikibooks:Sandbox is fine for
short-lived experiments.

http://shoutwiki.com can host MediaWiki pages.
http://jumpwiki.com/wiki/Main_Page can also host MediaWiki pages, but
there are troubles with align envirs and math (ugly typesetting and
some strange indents).

Create a user account, choose *Create a Wiki* in the menu on the left,
fill out the form, wait until you get a Main Page, click on edit, make
references to a new page, say [[First demo|demo]], save, click on
demo and fill out that page with the content of a mydoconcefile.wiki,
sometimes it is necessary to create a new account, just do that and
go back.
"""
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from builtins import range


import re, os, subprocess, sys, subprocess
from .common import default_movie, plain_exercise, insert_code_and_tex
from .plaintext import plain_quiz
from .misc import _abort
from .doconce import errwarn

def align2equations(math_text):
    """
    Transform an align environment to a set of equation environments.
    Used to handle multiple equations if align does not work well.

    Note: This version is outdated. common.align2equations is the
    newest attempt to implement align in terms of single equations.
    """
    if not '{align' in math_text:
        return
    math_text = math_text.replace('&', '')
    math_text = math_text.replace('\\\\', r"""
</math>

:<math>""")
    pattern = r'\\(begin|end)\{align\*?\}\s*'
    math_text = re.sub(pattern, '', math_text)
    # :<math> and </math> surroundings appear when !bt and !et are translated
    return math_text

def equation2nothing(math_text):
    pattern = r'\\(begin|end)\{equation\*?\}\s*'
    math_text = re.sub(pattern, '', math_text)
    math_text = math_text.replace(r'\[', '')
    math_text = math_text.replace(r'\]', '')
    return math_text

def remove_labels(math_text):
    pattern = 'label\{(.+?)\}\s*'
    labels = re.findall(pattern, math_text)
    if labels:
        math_text = re.sub(pattern, '', math_text)
    return math_text, labels


def mwiki_code(filestr, code_blocks, code_block_types,
               tex_blocks, format):
    # http://en.wikipedia.org/wiki/Help:Displaying_a_formula
    # MediaWiki math does not support labels in equations.
    # The enviros equation and \[ \] must be removed (not supported).

    for i in range(len(tex_blocks)):
        # Standard align works in Wikipedia and Wikibooks.
        # Standard align gives somewhat ugly output on wiiki.com services,
        # but a set of separate equations is not much better.
        # We therefore stick to align instead.
        #tex_blocks[i] = align2equations(tex_blocks[i])
        tex_blocks[i] = equation2nothing(tex_blocks[i])
        tex_blocks[i], labels = remove_labels(tex_blocks[i])
        for label in labels:
            if label in filestr:
                errwarn('*** warning: reference to label "%s" in an equation does not work in MediaWiki' % label)

    filestr = insert_code_and_tex(filestr, code_blocks, tex_blocks, format)

    # Supported programming languages:
    # http://www.mediawiki.org/wiki/Extension:SyntaxHighlight_GeSHi#Supported_languages
    envir2lang = dict(cod='python', pycod='python', cycod='python',
                      fcod='fortran', ccod='c', cppcod='cpp',
                      mcod='matlab', plcod='perl', shcod='bash',
                      pro='python', pypro='python', cypro='python',
                      fpro='fortran', cpro='c', cpppro='cpp',
                      mpro='matlab', plpro='perl', shpro='bash',
                      rbpro='ruby', rbcod='ruby',
                      javacod='java', javapro='java',
                      htmlcod='html5', xmlcod='xml',
                      htmlpro='html5', xmlpro='xml',
                      html='html5', xml='xml',
                      sys='bash', dat='text', csv='text', txt='text',
                      pyoptpro='python', pyscpro='python',
                      ipy='python', pyshell='python',
                      )

    for key in envir2lang:
        language = envir2lang[key]
        cpattern = re.compile(r'^!bc\s+%s\s*\n' % key, flags=re.MULTILINE)
        filestr = cpattern.sub('<syntaxhighlight lang="%s">\n' % \
                               envir2lang[key], filestr)
    c = re.compile(r'^!bc.*$\n', re.MULTILINE)
    filestr = c.sub('<syntaxhighlight lang="text">\n', filestr)
    filestr = re.sub(r'!ec\n', '</syntaxhighlight>\n', filestr)
    c = re.compile(r'^!bt\n', re.MULTILINE)
    filestr = c.sub(':<math>\n', filestr)
    filestr = re.sub(r'!et\n', '</math>\n', filestr)

    # Final fix of MediaWiki file

    # __TOC__ syntax is misinterpretated as paragraph heading, so we
    # use <<<TOC>>> instead and replace to right syntax here at the end.
    filestr = filestr.replace('<<<TOC>>>', '__TOC__')

    return filestr


def mwiki_figure(m):
    filename = m.group('filename')
    link = filename if filename.startswith('http') else None
    if not link and not os.path.isfile(filename):
        raise IOError('no figure file %s' % filename)

    basename  = os.path.basename(filename)
    stem, ext = os.path.splitext(basename)
    root, ext = os.path.splitext(filename)
    if link is None:
        if not ext in '.png .gif .jpg .jpeg'.split():
            # try to convert image file to PNG, using
            # convert from ImageMagick:
            cmd = 'convert %s png:%s' % (filename, root+'.png')
            try:
                output = subprocess.check_output(cmd, shell=True,
                                                 stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                errwarn('\n**** warning: could not run ' + cmd)
                errwarn('       convert %s to PNG format manually' % filename)
                _abort()
            filename = root + '.png'

    caption = m.group('caption').strip()
    if caption != '':
        caption = '|' + caption  # add | for non-empty caption
    else:
        # Avoid filename as caption when caption is empty
        # see http://www.mediawiki.org/wiki/Help:Images
        caption = '|<span title=""></span>'
    # keep label if it's there:
    caption = re.sub(r'label\{(.+?)\}', '(\g<1>)', caption)

    size = ''
    opts = m.group('options').strip()
    if opts:
        info = dict([s.split('=') for s in opts.split()])
        if 'width' in info and 'height' in info:
            size = '|%sx%spx' % (info['width'], info['height'])
        elif 'width' in info:
            size = '|%spx' % info['width']
        elif 'height' in info:
            size = '|x%spx' % info['height']

    if link:
        # We link to some image on the web
        filename = os.path.basename(filename)
        link = os.path.dirname(link)
        result = r"""
[[File:%s|frame%s|link=%s|alt=%s%s]]
""" % (filename, size, link, filename, caption)
    else:
        # We try to link to a file at wikimedia.org.
        found_wikimedia = False
        orig_filename = filename
        # Check if the file exists and find the appropriate wikimedia name.
        # http://en.wikipedia.org/w/api.php?action=query&titles=Image:filename&prop=imageinfo&format=xml

        # Skip directories - get the basename
        filename = os.path.basename(filename)
        import urllib.request, urllib.parse, urllib.error
        prms = urllib.parse.urlencode({
            'action': 'query', 'titles': 'Image:' + filename,
            'prop': 'imageinfo', 'format': 'xml'})
        url = 'http://en.wikipedia.org/w/api.php?' + prms
        try:
            errwarn(' ...checking if %s is stored at en.wikipedia.org/w/api.php...' % filename)
            f = urllib.request.urlopen(url)

            imageinfo = f.read()
            imageinfo = imageinfo.decode('utf-8')
            f.close()
            def get_data(name, text):
                pattern = '%s="(.*?)"' % name
                m = re.search(pattern, text)
                if m:
                    match = m.group(1)
                    if 'Image:' in match:
                        return match.split('Image:')[1]
                    if 'File:' in match:
                        return match.split('File:')[1]
                    else:
                        return match
                else:
                    return None

            data = ['from', 'to', 'title', 'missing', 'imagerepository',
                    'timestamp', 'user']
            orig_filename = filename
            filename = get_data('title', imageinfo)
            user = get_data('user', imageinfo)
            timestamp = get_data('timestamp', imageinfo)
            if user:
                found_wikimedia = True
                errwarn(' ...found %s at wikimedia' % filename)
                result = r"""
    [[File:%s|frame%s|alt=%s%s]] <!-- user: %s, filename: %s, timestamp: %s -->
    """ % (filename, size, filename, caption, user, orig_filename, timestamp)
        except IOError:
            errwarn(' ...no Internet connection...')

        if not found_wikimedia:
            errwarn(' ...for wikipedia/wikibooks you must upload image file %s to\n    common.wikimedia.org' % orig_filename)
            # see http://commons.wikimedia.org/wiki/Commons:Upload
            # and http://commons.wikimedia.org/wiki/Special:UploadWizard
            errwarn(' ...for now we use local file %s' % filename)
            # This is fine if we use github wiki

            result = r"""
[[File:%s|frame%s|alt=%s%s]] <!-- not yet uploaded to common.wikimedia.org -->
""" % (filename, size, filename, caption)

    return result

from .common import table_analysis


def mwiki_author(authors_and_institutions, auth2index,
                 inst2index, index2inst, auth2email):

    authors = []
    for author, i, email in authors_and_institutions:
        if email is None:
            email_text = ''
        else:
            name, adr = email.split('@')
            email_text = ' (%s at %s)' % (name, adr)
        authors.append('_%s_%s' % (author, email_text))

    if len(authors) ==  1:
        authors = authors[0]
    elif len(authors) == 2:
        authors = authors[0] + ' and ' + authors[1]
    elif len(authors) > 2:
        authors[-1] = 'and ' + authors[-1]
        authors = ', '.join(authors)
    else:
        # no authors:
        return ''
    text = '\n\nBy ' + authors + '\n\n'
    # we skip institutions in mwiki
    return text

from .gwiki import wiki_ref_and_label_common

def mwiki_ref_and_label(section_label2title, format, filestr):
    return wiki_ref_and_label_common(section_label2title, format, filestr)

def mwiki_admon(block, format, title='Warning', text_size='normal',
                admon_type='warning'):
    if title.lower().strip() == 'none':
        title = ''
    # Blocks without explicit title should have empty title
    if title == 'Block':
        title = ''

    if title and title[-1] not in ('.', ':', '!', '?'):
        # Make sure the title ends with puncuation
        title += '.'

    admon_type2mwiki = dict(notice='notice',
                            warning='warning',  # or critical or important
                            hint='notice',
                            quote='quote')
    if admon_type in admon_type2mwiki:
        admon_type = admon_type2mwiki[admon_type]  # use mwiki admon
    else:
        admon_type = title # Just use the title

    text = "'''%s''' " % title + block

    if text_size == 'normal':
        text_size = '90%'
    elif text_size == 'large':
        text_size = '130%'
    elif text_size == 'small':
        text_size = '80%'

    if admon_type == 'quote':
        s = """
{{quote box
| quote = %s
| textstyle = font-size: %s;
}}

""" % (block, text_size)
        # quote has also | source = ... but other formats like
        # latex and html have no specific source tag, so it must
        # be typeset manually
    else:
        s = """
{{mbox
| type = %s
| textstyle = font-size: %s;
| text = %s
}}

""" % (admon_type, text_size, text)
    return s

# mbox: notice

def define(FILENAME_EXTENSION,
           BLANKLINE,
           INLINE_TAGS_SUBST,
           CODE,
           LIST,
           ARGLIST,
           TABLE,
           EXERCISE,
           FIGURE_EXT,
           CROSS_REFS,
           INDEX_BIB,
           TOC,
           ENVIRS,
           QUIZ,
           INTRO,
           OUTRO,
           filestr):
    # all arguments are dicts and accept in-place modifications (extensions)

    FILENAME_EXTENSION['mwiki'] = '.mwiki'  # output file extension
    BLANKLINE['mwiki'] = '\n'

    # replacement patterns for substitutions of inline tags
    INLINE_TAGS_SUBST['mwiki'] = {
        'math':          r'\g<begin><math>\g<subst></math>\g<end>',
        'math2':         r'\g<begin><math>\g<latexmath></math>\g<end>',
        'emphasize':     r"\g<begin>''\g<subst>''\g<end>",
        'bold':          r"\g<begin>'''\g<subst>'''\g<end>",
        'verbatim':      r'\g<begin><code>\g<subst></code>\g<end>',
        #'linkURL':       r'\g<begin>[\g<url> \g<link>]\g<end>',
        'linkURL2':      r'[\g<url> \g<link>]',
        'linkURL3':      r'[\g<url> \g<link>]',
        'linkURL2v':     r'[\g<url> <code>\g<link></code>]',
        'linkURL3v':     r'[\g<url> <code>\g<link></code>]',
        'plainURL':      r'\g<url>',
        'colortext':     r'<font color="\g<color>">\g<text></font>',
        'chapter':       r"""== '''\g<subst>''' ==""",
        'section':       r'== \g<subst> ==',
        'subsection':    r'=== \g<subst> ===',
        'subsubsection': r'==== \g<subst> ====\n',
        'paragraph':     r"''\g<subst>''\n",
        'title':         r'#TITLE (actually governed by the filename): \g<subst>\n',
        'date':          r'===== \g<subst> =====',
        'author':        mwiki_author, #r'===== \g<name>, \g<institution> =====',
#        'figure':        r'<\g<filename>>',
        'figure':        mwiki_figure,
        'movie':         default_movie,  # will not work for HTML movie player
        'comment':       '<!-- %s -->',
        'abstract':      r'\n*\g<type>.* \g<text>\g<rest>',
        'linebreak':     r'\g<text><br />',
        'non-breaking-space': '&nbsp;',
        'horizontal-rule': '----',
        'ampersand2':    r' \g<1>&\g<2>',
        }

    CODE['mwiki'] = mwiki_code
    from .html import html_table
    TABLE['mwiki'] = html_table
    ENVIRS['mwiki'] = {
        'warning':   lambda block, format, title='Warning', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'warning'),
        'notice':    lambda block, format, title='Notice', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'notice'),
        'question':  lambda block, format, title='Question', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'question'),
        'hint':      lambda block, format, title='Hint', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'hint'),
        'summary':   lambda block, format, title='Summary', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'summary'),
        'block':     lambda block, format, title='Block', text_size='normal':
        mwiki_admon(block, format, title, text_size, 'block'),
        'box':       lambda block, format, title='none', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'box'),
        'quote':     lambda block, format, title='none', text_size='normal':
           mwiki_admon(block, format, title, text_size, 'quote'),
        }

    # native list:
    LIST['mwiki'] = {
        'itemize':     {'begin': '\n', 'item': '*', 'end': '\n\n'},
        'enumerate':   {'begin': '\n', 'item': '#', 'end': '\n\n'},
        'description': {'begin': '\n', 'item': '* %s ', 'end': '\n\n'},
        'separator': '\n'}
    # Try this:
    LIST['mwiki'] = LIST['html']


    # how to typeset description lists for function arguments, return
    # values, and module/class variables:
    ARGLIST['mwiki'] = {
        'parameter': '*argument*',
        'keyword': '*keyword argument*',
        'return': '*return value(s)*',
        'instance variable': '*instance variable*',
        'class variable': '*class variable*',
        'module variable': '*module variable*',
        }

    FIGURE_EXT['mwiki'] = {
        'search': ('.png', '.gif', '.jpg', '.jpeg'),
        'convert': ('.png', '.gif', '.jpg')}
    CROSS_REFS['mwiki'] = mwiki_ref_and_label
    from .plaintext import plain_index_bib
    EXERCISE['mwiki'] = plain_exercise
    INDEX_BIB['mwiki'] = plain_index_bib
    TOC['mwiki'] = lambda s, f: '<<<TOC>>>'  # __TOC__ will be wrongly translated to paragraph headline and needs a fix
    QUIZ['mwiki'] = plain_quiz

    # document start:
    INTRO['mwiki'] = ''



# -*- coding: utf-8 -*-
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import range
import os, sys
from subprocess import getstatusoutput
from doconce.sphinx import sphinx_dir
from doconce.misc import (
    misc_option,
    insertdocstr,
    spellcheck,
    capitalize,
    gwiki_figsubst,
    latin2html,
    remove_inline_comments,
    apply_inline_edits,
    sphinxfix_localURLs,
    old2new_format,
    latex_header,
    latex_footer,
    latex_problems,
    ref_external,
    find_nonascii_chars,
    replace_from_file,
    replace,
    subst,
    find,
    include_map,
    expand_mako,
    wildcard_notation,
    teamod,
    list_fig_src_files,
    list_labels,
    split_rst,
    slides_html,
    split_html,
    slides_beamer,
    slides_markdown,
    html_colorbullets,
    md2html,
    md2latex,
    change_encoding,
    guess_encoding,
    clean,
    lightclean,
    remove_exercise_answers,
    remove,
    grab,
    ptex2tex,
    include_map,
    expand_commands,
    combine_images,
    latex_exercise_toc,
    pygmentize,
    makefile,
    latex2doconce,
    latex_dislikes,
    html2doconce,
    ipynb2doconce,
    fix_bibtex4publish,
    diff,
    gitdiff,
    csv2table,
    extract_exercises,
    linkchecker,
    grep,
    help_format
    )

commands = "format, help, sphinx_dir, subst, replace, replace_from_file, clean, lightclean, spellcheck, find, ptex2tex, " \
           "guess_encoding, include_map, expand_commands, expand_mako, combine_images, change_encoding, capitalize, " \
           "find_nonascii_chars, gwiki_figsubst, md2html, md2latex, remove_inline_comments, apply_inline_edits, grab, " \
           "remove, remove_exercise_answers, split_rst, split_html, slides_html, slides_beamer, slides_markdown, " \
           "csv2table, extract_exercises, latin2html, grep, latex_header, latex_footer, latex_problems, ref_external, " \
           "html_colorbullets, list_fig_src_files, list_labels, teamod, sphinxfix_localURLs, latex_exercise_toc, " \
           "insertdocstr, old2new_format, linkchecker, latex2doconce, latex_dislikes, html2doconce, ipynb2doconce, " \
           "pygmentize, makefile, diff, gitdiff, fix_bibtex4publish".split(", ")

def help():
    _registered_commands = [
        ('%65s' % 'transform doconce file to another format', """doconce format html|latex|pdflatex|rst|sphinx|plain|gwiki|mwiki|\n               cwiki|pandoc|st|epytext dofile"""),

        (' '*29 + 'substitute a phrase by another using regular expressions (in this example -s is the re.DOTALL modifier, -m is the re.MULTILINE modifier, -x is the re.VERBOSE modifier, --restore copies backup files back again)',
            """doconce subst [-s -m -x --restore] regex-pattern \\ \n        regex-replacement file1 file2 ..."""),
        ('replace a phrase by another literally (exact text substitution)', """doconce replace from-text to-text file1 file2 ..."""),
        ('replace using from and to phrases from file', """doconce replace_from_file file-with-from-to-replacements file1 file2 ..."""),
        ('search for a (regular) expression in all .do.txt files in the current directory tree (useful when removing compilation errors)', """doconce find expression"""),
        ('print an overview of how various files are included in the root doc', """doconce include_map mydoc.do.txt"""),
        ('replace all mako function calls by the `results of the calls', """doconce expand_mako mako_code_file funcname file1 file2 ..."""),
        ('remove all inline comments in a doconce file', """doconce remove_inline_comments dofile"""),
        ('apply all edits specified through inline comments', """doconce apply_inline_edits"""),
        (' '*36 +   'create a directory for the sphinx format (requires sphinx version >= 1.1)',
            """doconce sphinx_dir copyright='John Doe' title='Long title' \\\n        short_title="Short title" version=0.1 intersphinx \\\n        /path/to/mylogo.png dofile"""),
        (' ' * 45 + 'split a sphinx/rst file into parts according to !split commands',
         """doconce format sphinx complete_file \ndoconce split_rst complete_file \ndoconce sphinx_dir complete_file \npython automake_sphinx.py"""),
        ('walk through a directory tree and insert doconce files as docstrings in *.p.py files', """doconce insertdocstr rootdir"""),
        ('remove all redundant files (keep source .do.txt and results: .pdf, .html, sphinx- dirs, .mwiki, .ipynb, etc.)', """doconce lightclean"""),
        ('remove all files that the doconce can regenerate', """doconce clean"""),
        ('change encoding', """doconce change_encoding utf-8 latin1 dofile"""),
        ('guess the encoding in a text', """doconce guess_encoding filename"""),
        ('find non-ascii characters in a file', """doconce find_nonascii_chars file1 file2 ..."""),
        ('split an html file into parts according to !split commands', """doconce split_html complete_file.html"""),
        ('create HTML slides from a (doconce) html file', """doconce slides_html slide_type complete_file.html"""),
        ('create LaTeX Beamer slides from a (doconce) latex/pdflatex file', """doconce slides_beamer complete_file.tex"""),
        ('create Remark slides from Markdown', """doconce slides_markdown complete_file.md remark --slide_style=light"""),
        ('replace bullets in lists by colored bullets', """doconce html_colorbullets file1.html file2.html ..."""),
        ('extract all exercises (projects and problems too)', """doconce extract_exercises tmp_mako__mydoc"""),
        ('grab selected text from a file', """doconce grab --from[-] from-text [--to[-] to-text] file > result"""),
        ('remove selected text from a file', """doconce remove --from[-] from-text [--to[-] to-text] file > result"""),
        ('list all figure, movie or included code files', """doconce grep FIGURE|MOVIE|CODE dofile"""),
        ('run spellcheck on a set of files', """doconce spellcheck [-d .mydict.txt] *.do.txt"""),
        ('   transform ptex2tex files (.p.tex) to ordinary latex file and manage the code environments', """doconce ptex2tex mydoc -DMINTED pycod=minted sys=Verbatim \\\n        dat=\\begin{quote}\\begin{verbatim};\end{verbatim}\end{quote}"""),
        ('make HTML file via pandoc from Markdown (.md) file', """doconce md2html file.md"""),
        ('make LaTeX file via pandoc from Markdown (.md) file', """doconce md2latex file.md"""),
        ('combine several images into one', """doconce combine_images image1 image2 ... output_file"""),
        ('report problems from a LaTeX .log file', """doconce latex_problems mydoc.log [overfull-hbox-limit]"""),
        ('list all figure files, movie files, and source code files needed', """doconce list_fig_src_files *.do.txt"""),
        ('list all labels in a document (for purposes of cleaning them up)', """doconce list_labels myfile"""),
        ('generate script for substituting generalized references', """doconce ref_external mydoc [pubfile]"""),
        ('check all links in HTML files', """doconce linkchecker *.html"""),
        ('change headings from "This is a Heading" to "This is a heading"', """doconce capitalize [-d .mydict.txt] *.do.txt"""),
        ('translate a latex document to doconce (requires usually manual fixing)', """doconce latex2doconce latexfile"""),
        ('check if there are problems with translating latex to doconce', """doconce latex_dislikes latexfile"""),
        ('translate an IPython/Jupyter notebook to doconce', """doconce ipynb2doconce notebookfile"""),
        ('typeset a doconce document with pygments (for pretty print of doconce itself)', """doconce pygmentize myfile [pygments-style]"""),
        ('generate a make.py script for translating a doconce file to various formats', """doconce makefile docname doconcefile [html sphinx pdflatex ...]"""),
        ('find differences between two files (diffprog can be difflib, diff, pdiff, latexdiff, kdiff3, diffuse, ...)', """doconce diff file1.do.txt file2.do.txt [diffprog]"""),
        ('find differences between the last two Git versions of several files', """doconce gitdiff file1 file2 file3 ..."""),
        ('convert csv file to doconce table format', """doconce csv2table somefile.csv"""),
        ('edit URLs to local files and place them in _static', """doconce sphinxfix_local_URLs file.rst"""),
        ('replace latex-1 (non-ascii) characters by html codes', """doconce latin2html file.html"""),
        ('fix common problems in bibtex files for publish import', """doconce fix_bibtex4publish file1.bib file2.bib ..."""),
        ('print the header (preamble) for latex file', """doconce latex_header"""),
        ('print the footer for latex files', """doconce latex_footer"""),
        ('expand short cut commands to full form in files', """doconce expand_commands file1 file2 ..."""),
        ('insert a table of exercises in a latex file myfile.p.tex', """doconce latex_exercise_toc myfile""")
    ]
    #doconce teamod name
    #doconce assemble name master.do.txt
    template = "\n\033[1m{0:70}\033[0m \033[94m{1:10}\033[0m"
    for help, command in _registered_commands:
        print(template.format(command, help))

def remove_options_from_command_line():
    """Remove options (--name) from sys.argv."""
    opts_index = [i for i in range(len(sys.argv))
                  if sys.argv[i].startswith('--')]
    for i in opts_index:
        del sys.argv[i]

def format():
    """
    Run the doconce module on a file (with extension .do.txt) in Doconce format
    and produce another format (latex, html, plain text, reStructuredText, ...)::

       doconce format html mydoc.do.txt
    """
    try:
        import doconce.doconce
    except ImportError:
        # use local doconce module in the doconce package source:
        try:
            thisdir = os.path.dirname(sys.argv[0])
            doconce_lib = os.path.join(thisdir, os.pardir, 'lib', 'doconce')
            sys.path.insert(0, doconce_lib)
            import doconce.doconce
            print('Successfull import of doconce locally')
        except ImportError as e:
            print(e)
            print('Could not import doconce from directory\n', os.getcwd())
            sys.exit(1)

    bg_session = doconce.doconce.format_driver()
    return bg_session

def main():
    from doconce import __version__ as version
    import doconce
    doconce_dir = os.path.dirname(doconce.__file__)
    if len(sys.argv) > 2 and sys.argv[1] == 'format' and misc_option('help'):
        help_format()
        sys.exit(0)
    elif len(sys.argv) == 1 or misc_option('help') and sys.argv[1] not in ['spellcheck']:
        print('DocOnce version', version, '(from %s)' % doconce_dir)
        print('Usage: doconce command [optional arguments]')
        print('commands: %s' % (' '.join(commands)))
        help()
        sys.exit(0)
    if misc_option('version'):
        print('DocOnce version', version, '(from %s)' % doconce_dir)
        sys.exit(0)

    command = sys.argv[1]
    del sys.argv[1]
    if command == '2format':
        command = 'format'
    if command == 'format':
        # For backward compatibility:
        if len(sys.argv) >= 2 and sys.argv[1] == 'LaTeX':
            sys.argv[1] = 'latex'
            print('\nWarning: Previous format LaTeX now has the name latex\n')
        if len(sys.argv) >= 2 and sys.argv[1] == 'HTML':
            sys.argv[1] = 'html'
            print('\nWarning: Previous format HTML now has the name html\n')

    # Treat wrong command name
    retval = None
    if command in commands:
        retval = eval(command + '()')
    else:
        if command in ('html', 'latex', 'sphinx', 'rst', 'plain', 'gwiki', 'mwiki', 'epydoc', 'pandoc'):
            print('command', command, 'is not a legal command for doconce, did you mean')
            print('doconce format %s %s?' % (command, ' '.join(sys.argv[1:])))
        else:
            print('command "%s" is not legal, must be among\n' % command)
            print(', '.join(commands))
    return retval

bg_session = main()
if bg_session is not None:
    print('...running bokeh bg_session.loop_until_closed() in the background')
    bg_session.loop_until_closed()



from builtins import str
from publish import config
_format_venue = config.formatting._format_venue
from publish.common import short_author
from publish.config.defaults import thesistype_strings
import re

#------------------------------------------------------------------------------
# DocOnce formatting
#------------------------------------------------------------------------------

def doconce_format_articles(paper):
    "Return string for article in DocOnce format"
    values = []

    # Key
    values.append(_doconce_get_key_string(paper))

    # Author
    values.append(_doconce_get_authors_string(paper["author"]))

    # Title
    values.append(_doconce_format_title(paper))

    # Journal
    formatted_venue = '*%s*' % \
         _format_venue(paper["journal"], paper["journal"], paper)

    values.append(formatted_venue)

    # Volume
    if "volume" in paper:
        vol = paper["volume"]
        if "number" in paper :
            vol += "(%s)" % paper["number"]
        values.append(vol)

    # Pages
    if "pages" in paper:
        values.append(_doconce_format_pages(paper["pages"]))

    # DOI
    if "doi" in paper:
        values.append(_doconce_format_doi(paper["doi"]))

    # arXiv
    if "arxiv" in paper:
        values.append(_doconce_format_arxiv(paper["arxiv"]))

    # Year
    if "year" in paper:
        values.append(paper["year"])

    if "note" in paper:
        values.append(_doconce_format_note(paper["note"]))

    # How published/URL
    values = _doconce_url(paper, values)

    # About sequence of URL vs year: For publications where howpublished
    # or URL is unusual, this info is placed last (articles, books, misc,
    # posters, theses, proceedings, chapters, edited books),
    # while for publications where howpublished/url is likely, it
    # is placed before year (talks, courses, manual, reports).

    return _doconce_join(values)

def doconce_format_books(paper):
    "Return string for book in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append('*%s*' % _doconce_format_title(paper))
    if "edition" in paper:
        values.append(_doconce_format_edition(paper))
    if "series" in paper:
        values.append(_doconce_format_bookseries(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    values.append(paper["year"])
    if "doi" in paper: values.append(_doconce_format_doi(paper["doi"]))
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_edited(paper):
    "Return string for edited book in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_chapters(paper):
    "Return string for chapter in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    values.append(paper["booktitle"])
    values.append(_doconce_format_editors(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    if "chapter" in paper:
        values.append("Chapter %s" % paper["chapter"])
    if "pages" in paper:
        values.append("pp. %s" % _doconce_format_pages(paper["pages"]))
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_proceedings(paper):
    "Return string for proceeding in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    values.append(_doconce_format_booktitle(paper))
    if "editor" in paper: values.append(_doconce_format_editors(paper))
    if "publisher" in paper: values.append(paper["publisher"])
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_reports(paper):
    "Return string for report in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "institution" in paper: values.append(_doconce_format_institution(paper))
    if "number" in paper: values.append(paper["number"])
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_manuals(paper):
    "Return string for manual in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    values = _doconce_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_theses(paper):
    "Return string for thesis in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    values.append(thesistype_strings[paper["thesistype"]])
    if "school" in paper: values.append(paper["school"])
    values = _doconce_url(paper, values)
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_courses(paper):
    "Return string for course in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "institution" in paper: values.append(_doconce_format_institution(paper))
    values = _doconce_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_talks(paper):
    "Return string for talk in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values = _doconce_url(paper, values)
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def doconce_format_posters(paper):
    "Return string for poster in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)


def doconce_format_misc(paper):
    "Return string for misc in DocOnce format"
    values = []
    values.append(_doconce_get_key_string(paper))
    if "author" in paper:
        values.append(_doconce_get_authors_string(paper["author"]))
    values.append(_doconce_format_title(paper))
    if "booktitle" in paper: values.append("in *%s*" % paper["booktitle"])
    if "meeting" in paper: values.append(paper["meeting"])
    if "thesistype" in paper: values.append(thesistype_strings[paper["thesistype"]])
    if "school" in paper: values.append(paper["school"])
    if "chapter" in paper: values.append("Chapter %s" % paper["chapter"])
    if "volume" in paper: values.append("vol. %s" % paper["volume"])
    if "pages" in paper: values.append("pp. %s" % _doconce_format_pages(paper["pages"]))
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_doconce_format_note(paper["note"]))
    values = _doconce_url(paper, values)
    return _doconce_join(values)

def _doconce_format_note(note):
    # Make sure \url is handled correctly, LaTeX needs URLs this way
    pattern = r'\\url\{(.+?)\}'
    m = re.search(pattern, note)
    if m:
        note = re.sub(pattern, 'URL: "\g<1>"', note)
    return note

def _doconce_url(paper, values):
    venue = None
    if "howpublished" in paper:
        venue = paper["howpublished"]
        if venue.startswith('\\url{') and venue.endswith('}'):
            venue = venue[5:-1]
        if venue.startswith('http'):
            venue = _doconce_format_url(venue)
        values.append(venue)
    if "url" in paper:
        if "note" in paper:
            if (paper["url"] not in paper["note"]) and \
               (paper["url"] not in paper["howpublished"]):
                values.append(_doconce_format_url(paper["url"]))
        else:
            values.append(_doconce_format_url(paper["url"]))
    return values

def _doconce_get_key_string(paper):
    return 'label{%s}' % paper["key"]

def _doconce_format_title(paper):
    "Format title for DocOnce, with or without link to PDF file"
    if paper["category"] == "courses":
        title = "%s (%s)" % (paper["title"], paper["code"])
    else:
        title = paper["title"]
    #return "*%s*" % title
    return title

def _doconce_format_booktitle(paper):
    return '*%s*' % paper["booktitle"]

def _doconce_format_edition(paper):
    return paper["edition"].lower() + ' edition'

def _doconce_format_bookseries(paper):
    return '*%s*' % paper["series"]

def _doconce_format_editors(paper):
    "Convert editor tuple to author string"
    return "edited by %s" % _doconce_get_authors_string(paper["editor"])

def _doconce_get_authors_string(authors):
    "Convert author tuple to author string"
    authors = [_doconce_mark_author(author, short_author(author).strip()) \
                   for author in authors]
    if len(authors) == 1:
        str = authors[0]
    else :
        if authors[-1] == "others":
            str =  ", ".join(authors[:-1]) + " et al."
        else:
            str = ", ".join(authors[:-1]) + " and " + authors[-1]
    #str = "_%s (%s)_" % (str, paper["year"])
    str = "_%s_" % (str)
    return str

def _doconce_mark_author(author, text) :
  "Mark the text with bold face if author is in the list of marked authors"
  if config.has_key("mark_author") and author.strip() in config.get("mark_author") :
    return "_%s_" % text
  else:
    return text

def _doconce_format_pages(pages):
    "Format pages"
    if "--" in pages:
        pages = pages.replace("--", "-")
    return "pp. %s" % pages

def _doconce_format_institution(paper):
    return '*%s*' % paper["institution"]

def _doconce_format_doi(doi):
    "Format DOI"
    return '"doi: %s": "http://dx.doi.org/%s"' % (doi, doi)

def _doconce_format_arxiv(arxiv):
    "Format arXiv"
    return '"arXiv: %s": "http://arxiv.org/abs/%s"' % (arxiv, arxiv)

def _doconce_format_url(url):
    "Format URL"
    if url.startswith('\\url{'):
        url = url[5:-1]
    if url.startswith('\\emph{'):
        url = url[6:-1]
    return 'URL: "%s"' % (url)

def _doconce_join(values):
    "Join values for DocOnce entry"
    entry = values[1] + ". \n    " + ",\n    ".join(values[2:]) + "." + "\n"
    entry = entry.replace("{", "")
    entry = entry.replace("}", "")
    entry = entry.replace("\\&", "&")
    entry = "  o " + values[0] + " " + entry
    return entry

doconce_format = {
    "articles"      : doconce_format_articles,
    "books"         : doconce_format_books,
    "edited"        : doconce_format_edited,
    "chapters"      : doconce_format_chapters,
    "proceedings"   : doconce_format_proceedings,
    "refproceedings": doconce_format_proceedings,
    "reports"       : doconce_format_reports,
    "manuals"       : doconce_format_manuals,
    "theses"        : doconce_format_theses,
    "courses"       : doconce_format_courses,
    "talks"         : doconce_format_talks,
    "posters"       : doconce_format_posters,
    "misc"          : doconce_format_misc}


#------------------------------------------------------------------------------
# reST formatting
#------------------------------------------------------------------------------

def rst_format_articles(paper):
    "Return string for article in reST format"
    values = []

    # Key
    values.append(_rst_get_key_string(paper))

    # Author
    values.append(_rst_get_authors_string(paper["author"]))

    # Title
    values.append(_rst_format_title(paper))

    # Journal
    try:
        formatted_venue = '*%s*' % \
           _format_venue(paper["journal"], paper["journal"], paper)
    except KeyError as e:
        raise ValueError('publish entry: %s\nmissing field: %s'
                         % (str(paper), str(e)))

    values.append(formatted_venue)

    # Volume
    if "volume" in paper:
        vol = paper["volume"]
        if "number" in paper :
            vol += "(%s)" % paper["number"]
        values.append(vol)

    # Pages
    if "pages" in paper:
        values.append(_rst_format_pages(paper["pages"]))

    # DOI
    if "doi" in paper:
        values.append(_rst_format_doi(paper["doi"]))

    # arXiv
    if "arxiv" in paper:
        values.append(_rst_format_arxiv(paper["arxiv"]))

    # Year
    if "year" in paper:
        values.append(paper["year"])

    # Note
    if "note" in paper:
        values.append(_rst_format_note(paper["note"]))

    # URL/howpublished
    values = _rst_url(paper, values)

    return _rst_join(values)

def rst_format_books(paper):
    "Return string for book in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append('*%s*' % _rst_format_title(paper))
    if "edition" in paper:
        values.append(_rst_format_edition(paper))
    if "series" in paper:
        values.append(_rst_format_bookseries(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    values.append(paper["year"])
    if "doi" in paper: values.append(_rst_format_doi(paper["doi"]))
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_edited(paper):
    "Return string for edited book in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    values.append(paper["year"])
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_chapters(paper):
    "Return string for chapter in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    values.append(paper["booktitle"])
    values.append(_rst_format_editors(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    if "chapter" in paper: values.append("Chapter %s" % paper["chapter"])
    if "pages" in paper: values.append("pp. %s" % _rst_format_pages(paper["pages"]))
    values.append(paper["year"])
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_proceedings(paper):
    "Return string for proceedings in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    values.append(paper["booktitle"])
    if "editor" in paper: values.append(_rst_format_editors(paper))
    if "publisher" in paper: values.append(paper["publisher"])
    values.append(paper["year"])
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_reports(paper):
    "Return string for report in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "institution" in paper: values.append(_rst_format_institution(paper))
    if "number" in paper: values.append(paper["number"])
    values = _rst_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_manuals(paper):
    "Return string for manual in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    values = _rst_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_theses(paper):
    "Return string for thesis in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    values.append(thesistype_strings[paper["thesistype"]])
    values.append(paper["school"])
    values.append(paper["year"])
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_courses(paper):
    "Return string for course in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "institution" in paper: values.append(_rst_format_institution(paper))
    values = _rst_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_talks(paper):
    "Return string for talk in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values = _rst_url(paper, values)
    values.append(paper["year"])
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def rst_format_posters(paper):
    "Return string for poster in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values.append(paper["year"])
    values = _rst_url(paper, values)
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)


def rst_format_misc(paper):
    "Return string for misc in reST format"
    values = []
    values.append(_rst_get_key_string(paper))
    if "author" in paper:
        values.append(_rst_get_authors_string(paper["author"]))
    values.append(_rst_format_title(paper))
    if "booktitle" in paper: values.append("in *%s*" % paper["booktitle"])
    if "meeting" in paper: values.append(paper["meeting"])
    if "thesistype" in paper: values.append(thesistype_strings[paper["thesistype"]])
    if "school" in paper: values.append(paper["school"])
    if "chapter" in paper: values.append("Chapter %s" % paper["chapter"])
    if "volume" in paper: values.append("vol. %s" % paper["volume"])
    if "pages" in paper: values.append("pp. %s" % _rst_format_pages(paper["pages"]))
    values = _rst_url(paper, values)
    if "year" in paper: values.append(paper["year"])
    if "note" in paper: values.append(_rst_format_note(paper["note"]))
    return _rst_join(values)

def _rst_format_note(note):
    # Make sure \url is handled correctly, LaTeX needs URLs this way
    pattern = r'\\url\{(.+?)\}'
    m = re.search(pattern, note)
    if m:
        note = re.sub(pattern, 'URL: "\g<1>"', note)
    return note

def _rst_url(paper, values):
    venue = None
    if "howpublished" in paper:
        venue = paper["howpublished"]
        if venue.startswith('\\url{') and venue.endswith('}'):
            venue = venue[5:-1]
        if venue.startswith('http'):
            venue = _rst_format_url(venue)
        values.append(venue)
    if "url" in paper:
        if "note" in paper:
            if (paper["url"] not in paper["note"]) and \
               (paper["url"] not in paper["howpublished"]):
                values.append(_rst_format_url(paper["url"]))
        else:
            values.append(_rst_format_url(paper["url"]))
    return values

def _rst_get_key_string(paper):
    return '[%s]' % paper["key"]

def _rst_format_title(paper):
    "Format title for reST, with or without link to PDF file"
    if paper["category"] == "courses":
        title = "%s (%s)" % (paper["title"], paper["code"])
    else:
        title = paper["title"]
    #return "*%s*" % title
    return title

def _rst_format_edition(paper):
    return paper["edition"].lower() + ' edition'

def _rst_format_bookseries(paper):
    return '*%s*' % paper["series"]

def _rst_format_editors(paper):
    "Convert editor tuple to author string"
    return "edited by %s" % _rst_get_authors_string(paper["editor"])

def _rst_get_authors_string(authors):
    "Convert author tuple to author string"
    authors = [_rst_mark_author(author, short_author(author).strip()) \
                   for author in authors]
    if len(authors) == 1:
        str = authors[0]
    else :
        if authors[-1] == "others":
            str =  ", ".join(authors[:-1]) + " et al."
        else:
            str = ", ".join(authors[:-1]) + " and " + authors[-1]
    #str = "_%s (%s)_" % (str, paper["year"])
    str = "_%s_" % (str)
    return str

def _rst_mark_author(author, text) :
  "Mark the text with bold face if author is in the list of marked authors"
  if config.has_key("mark_author") and author.strip() in config.get("mark_author") :
    return "_%s_" % text
  else:
    return text

def _rst_format_pages(pages):
    "Format pages"
    if "--" in pages:
        pages = pages.replace("--", "-")
    return "pp. %s" % pages

def _rst_format_institution(paper):
    return '*%s*' % paper["institution"]

def _rst_format_doi(doi):
    "Format DOI"
    return '"doi: %s": "http://dx.doi.org/%s"' % (doi, doi)

def _rst_format_arxiv(arxiv):
    "Format arXiv"
    return '"arXiv: %s": "http://arxiv.org/abs/%s"' % (arxiv, arxiv)

def _rst_format_url(url):
    "Format URL"
    return '`%s <%s>`_' % (url, url)

def _rst_join(values):
    "Join values for reST entry"
    entry = ".. " + values[0] + "\n   " + values[1] + ". " + ",\n   ".join(values[2:]) + "." + "\n"
    entry = entry.replace("{", "")
    entry = entry.replace("}", "")
    return entry

rst_format = {
    "articles"      : rst_format_articles,
    "books"         : rst_format_books,
    "edited"        : rst_format_edited,
    "chapters"      : rst_format_chapters,
    "proceedings"   : rst_format_proceedings,
    "refproceedings": rst_format_proceedings,
    "reports"       : rst_format_reports,
    "manuals"       : rst_format_manuals,
    "theses"        : rst_format_theses,
    "courses"       : rst_format_courses,
    "talks"         : rst_format_talks,
    "posters"       : rst_format_posters,
    "misc"          : rst_format_misc}


#------------------------------------------------------------------------------
# XML formatting
#------------------------------------------------------------------------------

def xml_format_articles(paper):
    "Return string for article in XML format"
    values = []

    # Key
    values.append(_xml('key', paper))

    # Author
    values.append(_xml_get_authors_string(paper["author"]))

    # Title
    values.append(_xml_format_title(paper))

    # Journal
    values.append('<journal>%s</journal>' %
                  _format_venue(paper["journal"], paper["journal"], paper))

    for tag in ['volume', 'number', 'pages', 'doi', 'arxiv', 'year', 'url']:
        if tag in paper:
            values.append(_xml(tag, paper))

    return _xml_join(values)

def xml_format_books(paper):
    "Return string for book in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "edition" in paper:
        values.append(_xml_format_edition(paper))
    if "series" in paper:
        values.append(_xml_format_bookseries(paper))
    if "publisher" in paper:
        values.append(_xml("publisher", paper))
    values.append(_xml("year", paper))
    if "doi" in paper: values.append(_xml('doi', paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_edited(paper):
    "Return string for edited book in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "publisher" in paper:
        values.append(paper["publisher"])
    values.append(paper["year"])
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_chapters(paper):
    "Return string for chapter in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    values.append(_xml("booktitle", paper))
    values.append(_xml_format_editors(paper))
    if "publisher" in paper:
        values.append(_xml("publisher", paper))
    if "chapter" in paper:
        values.append(_xml("chapter", paper))
    if "pages" in paper:
        values.append(_xml_format_pages(paper["pages"]))
    values.append(_xml('year', paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_proceedings(paper):
    "Return string for proceeding in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    values.append(_xml_format_booktitle(paper))
    if "editor" in paper: values.append(_xml_format_editors(paper))
    if "publisher" in paper: values.append(_xml("publisher", paper))
    values.append(_xml("year", paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_reports(paper):
    "Return string for report in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "institution" in paper: values.append(_xml_format_institution(paper))
    if "number" in paper: values.append(_xml("number", paper))
    values.append(_xml("year", paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_manuals(paper):
    "Return string for manual in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "year" in paper: values.append(_xml("year", paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_theses(paper):
    "Return string for thesis in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    values.append(thesistype_strings[paper["thesistype"]])
    if "school" in paper: values.append(_xml("school", paper))
    values.append(_xml("year", paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_courses(paper):
    "Return string for course in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "institution" in paper: values.append(_xml('institution', paper))
    if "year" in paper: values.append(_xml('year', paper))
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_talks(paper):
    "Return string for talk in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values.append(paper["year"])
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def xml_format_posters(paper):
    "Return string for poster in XML format"
    values = []
    values.append(_xml('key', paper))
    values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "meeting" in paper: values.append(paper["meeting"])
    values.append(paper["year"])
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)


def xml_format_misc(paper):
    "Return string for misc in XML format"
    values = []
    values.append(_xml('key', paper))
    if "author" in paper:
        values.append(_xml_get_authors_string(paper["author"]))
    values.append(_xml_format_title(paper))
    if "howpublished" in paper:
        howpublished = paper["howpublished"]
        values.append(howpublished)
    if "booktitle" in paper: values.append("in *%s*" % paper["booktitle"])
    if "meeting" in paper: values.append(paper["meeting"])
    if "thesistype" in paper: values.append(thesistype_strings[paper["thesistype"]])
    if "school" in paper: values.append(paper["school"])
    if "chapter" in paper: values.append("Chapter %s" % paper["chapter"])
    if "volume" in paper: values.append("vol. %s" % paper["volume"])
    if "pages" in paper: values.append("pp. %s" % _xml_format_pages(paper["pages"]))
    if "year" in paper: values.append(paper["year"])
    if "url" in paper: values.append(_xml('url', paper))
    return _xml_join(values)

def _xml(key, paper):
    return '<%s>%s</%s>' % (key, paper[key], key)

def _xml_format_title(paper):
    "Format title for XML, with or without link to PDF file"
    if paper["category"] == "courses":
        title = "<title>%s (%s)</title>" % (paper["title"], paper["code"])
    else:
        title = '<title>%s</title>' % paper["title"]
    return title

def _xml_format_institution(paper):
    return '*%s*' % paper["institution"]

def _xml_format_booktitle(paper):
    return '*%s*' % paper["booktitle"]

def _xml_format_edition(paper):
    return paper["edition"].lower() + ' edition'

def _xml_format_bookseries(paper):
    return '*%s*' % paper["series"]

def _xml_format_editors(paper):
    "Convert editor tuple to author string"
    return "edited by %s" % _xml_get_authors_string(paper["editor"])

def _xml_get_authors_string(authors):
    "Convert author tuple to author string"
    authors = [_xml_mark_author(author, short_author(author).strip())
               for author in authors]
    return '\n'.join(authors)

def _xml_mark_author(author, text):
  "Mark the text with bold face if author is in the list of marked authors"
  if config.has_key("mark_author") and author.strip() in config.get("mark_author") :
    return '<author marked="True">%s</author>' % text
  else:
    return '<author marked="False">%s</author>' % text

def _xml_format_pages(pages):
    if "--" in pages:
        pages = pages.replace("--", "-")
    return '<pages>%s</pages>' % pages

def _xml_join(values):
    return '\n' + '\n'.join(values) + '\n'

xml_format = {
    "articles"      : xml_format_articles,
    "books"         : xml_format_books,
    "edited"        : xml_format_edited,
    "chapters"      : xml_format_chapters,
    "proceedings"   : xml_format_proceedings,
    "refproceedings": xml_format_proceedings,
    "reports"       : xml_format_reports,
    "manuals"       : xml_format_manuals,
    "theses"        : xml_format_theses,
    "courses"       : xml_format_courses,
    "talks"         : xml_format_talks,
    "posters"       : xml_format_posters,
    "misc"          : xml_format_misc}



#!/usr/bin/env python
from __future__ import print_function
from __future__ import absolute_import
from builtins import range
from past.builtins import basestring
import shutil, re, sys, os

# NOTE: all newcommands can only span *one line*!
# (necessary requirement since a findall with re.DOTALL will
# not catch the final } of a command, real parsing is then neeeded)

def process_newcommand(line):
    line = line.replace('renewcommand', 'newcommand') # make syntax uniform
    # newcommand without arguments:
    pattern1 = r'\\newcommand\{(.+?)\}\s*\{(.*)\}'
    found = False
    m = re.search(pattern1, line)
    if m:
        # for a newcommand \x we can have many appearances:
        # \x |{\x}|\x{}|(\x)|\x, and replacing \x in \xpoint must
        # but avoided - the idea is to use the regex (\x)([^A-Za-z])
        end_pattern = r'([^A-Za-z])'
        pattern = m.group(1) + end_pattern
        text = m.group(2)
        if not text.endswith('}'):
            # Do not encapsulate plain words in {}
            if not re.search(r'[A-Za-z ,.]{%d}' % len(text), text):
                text = '{' + text + '}'
        replacement = text + r'\g<1>'  # \g<1> is the end_pattern
        # could also use a lookahead pattern: (?=[^A-Za-z]), not tested
        #pattern = m.group(1) + r'(?=[^A-Za-z])'
        #replacement = m.group(2)
        found = True

    # newcommand with arguments:
    pattern2 = r'\\newcommand\{(.+)\}\[(\d)\]\{(.*)\}'
    m = re.search(pattern2, line)
    if m:
        nargs = int(m.group(2))
        args = r'\{(.+?)\}'*nargs
        pattern = m.group(1) + args
        replacement = m.group(3)
        if not replacement.endswith('}'):
            # Do not encapsulate plain words in {}
            if not re.search(r'[A-Za-z ,.]{%d}' % len(replacement), replacement):
                replacement = '{' + replacement + '}'
        for i in range(1, nargs+1):
            replacement = replacement.replace('#%d' % i, r'\g<%d>' % i)
        found = True

    if found:
        # fix \x, \b, \r... etc in strings
        from .latex import fix_latex_command_regex as fix
        pattern = fix(pattern, 'match')
        replacement = fix(replacement, 'replacement')
        return pattern, replacement

def parse_newcommands(filename):
    f = open(filename, 'r')
    lines = f.readlines()
    f.close()
    newcommands = []

    for line in lines:
        line = line.strip()
        if line.startswith('%') or line == '':
            continue
        elif line.startswith(r'\newcommand') or \
             line.startswith(r'\renewcommand'):
            pattern, replacement = process_newcommand(line)
            newcommands.append((pattern, replacement))
        else:
            raise SyntaxError(
                '*** error: illegal line\n  %s\n    line' % line +
                'must start with %% or ' + r'\newcommand')
    return newcommands

def substitute(source, newcommands):
    """
    Expand all newcommands in the list 'newcommands' of
    (pattern, replacement) pairs. source can be a filename
    or just a string with text. If source is a filename, a backup
    file with extension .old~ is first made and then the original
    file is overwritten by the new text with expanded commands.
    """
    if os.path.isfile(source):
        shutil.copy(source, source + '.old~')
        f = open(source, 'r')
        text = f.read()
        f.close()
    else:
        text = source

    from .doconce import debugpr
    for pattern, replacement in newcommands:
        text, n = re.subn(pattern, replacement, text)
        if n:
            debugpr('newcommand replacement: %s -> %s (%d times)'
                    % (pattern, replacement, n))

    # newcommands can be nested, let's repeat
    depth = 2
    for i in range(depth):
        for pattern, replacement in newcommands:
            m = re.search(pattern, text)
            text, n = re.subn(pattern, replacement, text)
            if n:
                debugpr('newcommand replacement: %s -> %s (%d times)'
                        % (pattern, replacement, n))

    if os.path.isfile(source):
        f = open(source, 'w')
        f.write(text)
        f.close()
    else:
        return text

def expand_newcommands(newcommands_files, source):
    if isinstance(newcommands_files, basestring):
        newcommands_files = [newcommands_files]  # always list
    newcommands = []
    for filename in newcommands_files:
        if os.path.isfile(filename):
            newcommands.extend(parse_newcommands(filename))
    #import pprint; pprint.pprint(newcommands)
    return substitute(source, newcommands)

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('Usage: expand_newcommands.py newcommands_file source_file1 source_file2 ...')
        sys.exit(1)
    newcommands_file = sys.argv[1]
    for source_file in sys.argv[2:]:
        print('expanding newcommands in', source_file)
        expand_newcommands(newcommands_file, source_file)
